<?php

declare(strict_types=1);

namespace mirzaev\accounts\tests;

use PHPUnit\Framework\TestCase;

use mirzaev\accounts\vk as account;

use GuzzleHttp\Client as browser;

/**
 * @testdox ВКонтакте-аккаунт
 */
final class vkTest extends TestCase
{
    use settings;

    /**
     * Аккаунт
     */
    private account $account;

    /**
     * Инициализация теста
     */
    public function setUp(): void
    {
        if (empty($this->id)) {
            $this->markTestSkipped('Не установлен идентификатор');
        }

        // Инициализация аккаунта
        $this->account = new account($this->id, __DIR__ . DIRECTORY_SEPARATOR . 'accounts');

        // Инициализация свойства SSL-протокола
        $this->account->ssl = $this->ssl;

        // Проверка
        $this->assertInstanceOf(account::class, $this->account, 'Не удалось создать аккаунт');
    }

    /**
     * Деинициализация теста
     */
    public function tearDown(): void
    {
        // Деаутентификация
        $this->account->deauth();

        $path = $this->account->path;

        // Удаление аккаунта
        unset($this->account);

        // Проверка
        $this->assertNull($this->account ?? null, 'Не удалось удалить аккаунт');

        // Удаление файлов
        unlink($path . DIRECTORY_SEPARATOR . 'cookie.txt');
        rmdir($path);

        // Проверка
        $this->assertFileDoesNotExist($path, 'Не удалось удалить директорию для тестов');

    }

    /**
     * @testdox Аутентификация (базовая) в мобильном режиме
     */
    public function testVkAuthBasicModeMobile(): void
    {
        if (empty($this->login)) {
            $this->markTestSkipped('Не установлен логин');
        } else if (empty($this->password)) {
            $this->markTestSkipped('Не установлен пароль');
        }

        // Подождать во избежание блокировки
        sleep(3);

        // Аутентификация
        $this->account->auth($this->login, $this->password);

        // Проверка
        $this->assertStringNotContainsString('Страница удалена либо ещё не создана.', (string) $this->account->browser->request('GET', 'https://m.vk.com/id0')->getBody(), 'Не удалось аутентифицироваться');
    }

    /**
     * @testdox Реаутентификация (базовая) в мобильном режиме
     */
    public function testVkReauthBasicModeMobile(): void
    {
        if (empty($this->login)) {
            $this->markTestSkipped('Не установлен логин');
        } else if (empty($this->password)) {
            $this->markTestSkipped('Не установлен пароль');
        }

        // Подождать во избежание блокировки
        sleep(3);

        // Аутентификация
        $this->account->auth($this->login, $this->password);

        // Проверка
        $this->assertStringNotContainsString('Страница удалена либо ещё не создана.', (string) $this->account->browser->request('GET', 'https://m.vk.com/id0')->getBody(), 'Не удалось аутентифицироваться');

        // Проверка выброса исключения
        $this->expectExceptionMessage('Повторная аутентификация запрещена');

        // Аутентификация
        $this->account->auth($this->login, $this->password);
    }

    /**
     * @testdox Аутентификация (базовая) в мобильном режиме с неправильным логином
     *
     * Тест может завериться неудачей, если ВКонтакте выдаст блокировку (надо повторить тест позже)
     */
    public function testVkAuthBasicModeMobileWhenLoginIncorrect(): void
    {
        // Подождать во избежание блокировки
        sleep(3);

        // Проверка выброса исключения
        $this->expectExceptionMessage('ВКонтакте: "Пожалуйста, проверьте правильность введённых данных."');

        // Аутентификация
        $this->account->auth('admin' . rand(0, 1000) . '@hood.su', (string) rand(0, 1000));
    }

    /**
     * @testdox Аутентификация (базовая) в мобильном режиме c генерацией ключа
     */
    public function testVkAuthBasicModeMobileWithKeyGeneration(): void
    {
        if (empty($this->login)) {
            $this->markTestSkipped('Не установлен логин');
        } else if (empty($this->password)) {
            $this->markTestSkipped('Не установлен пароль');
        }

        // Подождать во избежание блокировки
        sleep(3);

        // Аутентификация
        $this->account->auth($this->login, $this->password);

        // Проверка
        $this->assertStringNotContainsString('Страница удалена либо ещё не создана.', (string) $this->account->browser->request('GET', 'https://m.vk.com/id0')->getBody(), 'Не удалось аутентифицироваться');

        // Генерация ключа
        $this->account->key($this->project_id);

        // Проверка
        $this->assertNotNull($this->account->key(), 'Не удалось сгенерировать ключ');
    }

    /**
     * @testdox Запись идентификатора
     */
    public function testSetId(): void
    {
        // Проверка выброса исключения
        $this->expectExceptionMessage('Запрещено инициализировать идентификатор');

        // Запись
        $this->account->id = rand(0, 100);
    }

    /**
     * @testdox Запись входного
     */
    public function testSetLogin(): void
    {
        if (empty($this->login)) {
            $this->markTestSkipped('Не установлен логин');
        }

        // Проверка выброса исключения
        $this->expectExceptionMessage('Запрещено инициализировать входной');

        // Запись
        $this->account->login = $this->login;
    }

    /**
     * @testdox Запись пароля
     */
    public function testSetPassword(): void
    {
        if (empty($this->password)) {
            $this->markTestSkipped('Не установлен пароль');
        }

        // Проверка выброса исключения
        $this->expectExceptionMessage('Запрещено инициализировать пароль');

        // Запись
        $this->account->password = $this->password;
    }

    /**
     * @testdox Запись браузера
     */
    public function testSetBrowser(): void
    {
        // Проверка выброса исключения
        $this->expectExceptionMessage('Запрещено инициализировать браузер');

        // Запись
        $this->account->browser = new browser();
    }

    /**
     * @testdox Запись корневой директории
     */
    public function testSetPath(): void
    {
        // Изначальное значение
        $old = $this->account->path;

        // Запись
        $this->account->path = __DIR__ . DIRECTORY_SEPARATOR . 'accounts' . DIRECTORY_SEPARATOR . 'test_directory';

        // Проверка
        $this->assertNotSame($this->account->path, $old, 'Не удалось записать корневую директорию');
    }

    /**
     * @testdox Запись SSL
     */
    public function testSetSsl(): void
    {
        // Изначальное значение
        $old = $this->account->ssl;

        // Запись
        $this->account->ssl = !$old;

        // Проверка
        $this->assertNotSame($this->account->ssl, $old, 'Не удалось записать SSL');
    }
}
